﻿-------------------------------------------------------
-- fyhcslb, 2016/08/10
-------------------------------------------------------
local _, addonTable = ...
local L = addonTable.L
local CURRENT_NAME, CURRENT_SERVER
local LRI = LibStub:GetLibrary("LibRealmInfo")	-- LibRealmInfo by Phanx

-------------------------------------------------------
-- menuitems
-------------------------------------------------------
UnitPopupButtons["GUILD_INVITE"] = {
	text = L.GUILD_INVITE,
	value = "GUILD_INVITE",
	dist = 0
}

UnitPopupButtons["COPY_NAME"] = {
	text = L.COPY_NAME,
	value = "COPY_NAME",
	dist = 0
}

UnitPopupButtons["ARMORY_URL"] = {
	text = L.ARMORY_URL,
	value = "ARMORY_URL",
	dist = 0
}

-------------------------------------------------------
-- "which" for specific button
-------------------------------------------------------
EnhancedMenu_ButtonSet = {}

EnhancedMenu_ButtonSet["GUILD_INVITE"] = {
	["PARTY"] = true,
	["PLAYER"] = true,
	["RAID_PLAYER"] = true,
	["FRIEND"] = true,
	["FRIEND_OFFLINE"] = true,
	["CHAT_ROSTER"] = true
}

EnhancedMenu_ButtonSet["COPY_NAME"] = {
	["PARTY"] = true,
	["PLAYER"] = true,
	["RAID_PLAYER"] = true,
	["FRIEND"] = true,
	["FRIEND_OFFLINE"] = true,
	["CHAT_ROSTER"] = true,
	["GUILD"] = true,
	["GUILD_OFFLINE"] = true
}

EnhancedMenu_ButtonSet["ARMORY_URL"] = {
	["PARTY"] = true,
	["PLAYER"] = true,
	["RAID_PLAYER"] = true,
	["FRIEND"] = true,
	["FRIEND_OFFLINE"] = true,
	["CHAT_ROSTER"] = true,
	["GUILD"] = true,
	["GUILD_OFFLINE"] = true
}

-- add button(s) to menu
for buttonName, buttonTable in pairs(EnhancedMenu_ButtonSet) do
	for which, _ in pairs(buttonTable) do
		table.insert(UnitPopupMenus[which], #(UnitPopupMenus[which]), buttonName)
	end
end

-------------------------------------------------------
-- util (FriendsMenuXP and enhanced)
-------------------------------------------------------
local function urlencode(obj)
    local currentIndex = 1;
    local charArray = {}
    while currentIndex <= #obj do
        local char = string.byte(obj, currentIndex);
        charArray[currentIndex] = char
        currentIndex = currentIndex + 1
    end
    local converchar = "";
    for _, char in ipairs(charArray) do
        converchar = converchar..string.format("%%%X", char)
    end
    return converchar;
end

local function showArmoryURL(name, server)
	if not name or name == "" or not server or server == "" then return end
	local portal = string.lower(GetCVar("portal"))
	if  portal ~= "cn" and portal ~= "kr" then
		local _, realm, _, _, _, _, region = LRI:GetRealmInfo(server)
		if not realm or realm == "" or not region or region == "" then
			DEFAULT_CHAT_FRAME:AddMessage("|cffff0000EnhancedMenu ERROR|r: Region not found! Use current region instead. Make it correct.")
			portal = LRI:GetCurrentRegion() -- not found, use current region
		else
			portal = string.lower(region)
		end
	end
	local host = ("http://%s.battle.net/"):format(portal)
	local armory = host.."wow/character/"..urlencode(server).."/"..urlencode(name).."/advanced"
	-- local armoryNoEncode = host.."wow/character/"..server.."/"..name.."/advanced"
	
	local editBox = ChatEdit_ChooseBoxForSend()
	ChatEdit_ActivateChat(editBox)
	editBox:SetText(armory)
	editBox:SetCursorPosition(0)
	editBox:HighlightText()
end

-------------------------------------------------------
-- hook
-------------------------------------------------------
hooksecurefunc("UnitPopup_ShowMenu", function(self, which)
	-- texplore("UnitPopupMenu", self)
    if (EnhancedMenu_ButtonSet["COPY_NAME"][which] or EnhancedMenu_ButtonSet["GUILD_INVITE"][which]) and UIDROPDOWNMENU_MENU_LEVEL == 1 then
        CURRENT_NAME, CURRENT_SERVER = self.name, self.server or GetRealmName()
    end
end)

hooksecurefunc("UnitPopup_OnClick", function(self)
	-- texplore("UnitPopup_OnClick", self)
    local name, server = UIDROPDOWNMENU_INIT_MENU.name, UIDROPDOWNMENU_INIT_MENU.server
	-- print("name:" .. (name or "nil") .. ", CURRENT_NAME:" .. (CURRENT_NAME or "nil"))
	-- print("server:" .. (server or "nil") .. ", CURRENT_SERVER:" .. (CURRENT_SERVER or "nil"))
    if name == CURRENT_NAME and not server then server = CURRENT_SERVER end
	
	if self.value == "GUILD_INVITE" then
		GuildInvite(name .. "-" .. server)
	elseif self.value == "COPY_NAME" then  -- FriendsMenuXP
		name = name .. "-" .. server
		if ( SendMailNameEditBox and SendMailNameEditBox:IsVisible() ) then
            SendMailNameEditBox:SetText(name)
            SendMailNameEditBox:HighlightText()
        else
            local editBox = ChatEdit_ChooseBoxForSend()
            if editBox:HasFocus() then
                editBox:Insert(name)
            else
                ChatEdit_ActivateChat(editBox)
                editBox:SetText(name)
                editBox:HighlightText()
            end
        end
	elseif self.value == "ARMORY_URL" then
		showArmoryURL(name, server)
	end
end)

-------------------------------------------------------
-- Alt + LeftButton = Invite
-- also from FriendsMenuXP
-------------------------------------------------------
function GetNameFromLink(link)
    local _, name, _ = strsplit(":", link)
    if ( name and (strlen(name) > 0) ) then	-- necessary?
        name = gsub(name, "([^%s]*)%s+([^%s]*)%s+([^%s]*)", "%3")
        name = gsub(name, "([^%s]*)%s+([^%s]*)", "%2")
    end
    return name
end

function EnhancedMenu_ChatFrame_OnHyperlinkShow(self, playerString, text, button)
    if(playerString and strsub(playerString, 1, 6) == "player") then
        if IsAltKeyDown() and button == "LeftButton" then
			DEFAULT_CHAT_FRAME.editBox:Hide()
            InviteUnit(GetNameFromLink(playerString))
            return
        end
    end
end

hooksecurefunc("ChatFrame_OnHyperlinkShow", EnhancedMenu_ChatFrame_OnHyperlinkShow)

-------------------------------------------------------
--- Premade ARMORY_URL & COPY_NAME
-------------------------------------------------------
EnhancedMenu_Premade = {}
EnhancedMenu_Premade["ARMORY_URL"] = {
	text = L.ARMORY_URL,
	func = function(_, name) showArmoryURL(string.split("-", name)) end,
	notCheckable = true,
	arg1 = nil, --Leader name goes here
	disabled = nil, --Disabled if we don't have a leader name yet
}

--if necessary, add it later
EnhancedMenu_Premade["COPY_NAME"] = {
	text = L.COPY_NAME,
	func = function(_, name) 
		local editBox = ChatEdit_ChooseBoxForSend()
		if editBox:HasFocus() then
			editBox:Insert(name)
		else
			ChatEdit_ActivateChat(editBox)
			editBox:SetText(name)
			editBox:HighlightText()
		end
	end,
	notCheckable = true,
	arg1 = nil, --Leader name goes here
	disabled = nil, --Disabled if we don't have a leader name yet
}

-- Interface\FrameXML\LFGList.lua line 2687
local LFG_LIST_SEARCH_ENTRY_MENU = {
    {
        text = nil, --Group name goes here
        isTitle = true,
        notCheckable = true,
    },
    {
        text = WHISPER_LEADER,
        func = function(_, name) ChatFrame_SendTell(name); end,
        notCheckable = true,
        arg1 = nil, --Leader name goes here
        disabled = nil, --Disabled if we don't have a leader name yet or you haven't applied
        tooltipWhileDisabled = 1,
        tooltipOnButton = 1,
        tooltipTitle = nil, --The title to display on mouseover
        tooltipText = nil, --The text to display on mouseover
    },
	{	-- ARMORY_URL
        text = EnhancedMenu_Premade["ARMORY_URL"].text,
		func = EnhancedMenu_Premade["ARMORY_URL"].func,
		notCheckable = EnhancedMenu_Premade["ARMORY_URL"].notCheckable,
		arg1 = EnhancedMenu_Premade["ARMORY_URL"].arg1,
        disabled = EnhancedMenu_Premade["ARMORY_URL"].disabled,
    },
    {
        text = LFG_LIST_REPORT_GROUP_FOR,
		hasArrow = true,
		notCheckable = true,
		menuList = {
			{
				text = LFG_LIST_BAD_NAME,
				func = function(_, id) C_LFGList.ReportSearchResult(id, "lfglistname"); end,
				arg1 = nil, --Search result ID goes here
				notCheckable = true,
			},
			{
				text = LFG_LIST_BAD_DESCRIPTION,
				func = function(_, id) C_LFGList.ReportSearchResult(id, "lfglistcomment"); end,
				arg1 = nil, --Search reuslt ID goes here
				notCheckable = true,
				disabled = nil,	--Disabled if the description is just an empty string
			},
			{
				text = LFG_LIST_BAD_VOICE_CHAT_COMMENT,
				func = function(_, id) C_LFGList.ReportSearchResult(id, "lfglistvoicechat"); end,
				arg1 = nil, --Search reuslt ID goes here
				notCheckable = true,
				disabled = nil,	--Disabled if the description is just an empty string
			},
			{
				text = LFG_LIST_BAD_LEADER_NAME,
				func = function(_, id) C_LFGList.ReportSearchResult(id, "badplayername"); end,
				arg1 = nil, --Search reuslt ID goes here
				notCheckable = true,
				disabled = nil,	--Disabled if we don't have a name for the leader
			},
		},
    },
    {
        text = CANCEL,
        notCheckable = true,
    },
};

-- Interface\FrameXML\LFGList.lua line 2744
function LFGListUtil_GetSearchEntryMenu(resultID)
    local id, activityID, name, comment, voiceChat, iLvl, honorLevel, age, numBNetFriends, numCharFriends, numGuildMates, isDelisted, leaderName, numMembers = C_LFGList.GetSearchResultInfo(resultID);
    local _, appStatus, pendingStatus, appDuration = C_LFGList.GetApplicationInfo(resultID);
    LFG_LIST_SEARCH_ENTRY_MENU[1].text = name;
    LFG_LIST_SEARCH_ENTRY_MENU[2].arg1 = leaderName;
    LFG_LIST_SEARCH_ENTRY_MENU[2].disabled = not leaderName;
	
	-- remove requirement "applied"
	-- local applied = (appStatus == "applied" or appStatus == "invited");
	-- LFG_LIST_SEARCH_ENTRY_MENU[2].disabled = not leaderName or not applied;
	-- LFG_LIST_SEARCH_ENTRY_MENU[2].tooltipTitle = (not applied) and WHISPER
	-- LFG_LIST_SEARCH_ENTRY_MENU[2].tooltipText = (not applied) and LFG_LIST_MUST_SIGN_UP_TO_WHISPER;
	
	-------------------------------------------------------
	-- add our menu(s)
    LFG_LIST_SEARCH_ENTRY_MENU[3].arg1 = leaderName;
    LFG_LIST_SEARCH_ENTRY_MENU[3].disabled = not leaderName;
	-------------------------------------------------------
	
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[1].arg1 = resultID;
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[2].arg1 = resultID;
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[2].disabled = (comment == "");
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[3].arg1 = resultID;
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[3].disabled = (voiceChat == "");
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[4].arg1 = resultID;
    LFG_LIST_SEARCH_ENTRY_MENU[4].menuList[4].disabled = not leaderName;
    return LFG_LIST_SEARCH_ENTRY_MENU;
end

-- Interface\FrameXML\LFGList.lua line 2763
local LFG_LIST_APPLICANT_MEMBER_MENU = {
    {
        text = nil, --Player name goes here
        isTitle = true,
        notCheckable = true,
    },
    {
        text = WHISPER,
        func = function(_, name) ChatFrame_SendTell(name); end,
        notCheckable = true,
        arg1 = nil, --Player name goes here
        disabled = nil, --Disabled if we don't have a name yet
    },
    {	-- ARMORY_URL
        text = EnhancedMenu_Premade["ARMORY_URL"].text,
		func = EnhancedMenu_Premade["ARMORY_URL"].func,
		notCheckable = EnhancedMenu_Premade["ARMORY_URL"].notCheckable,
		arg1 = EnhancedMenu_Premade["ARMORY_URL"].arg1,
        disabled = EnhancedMenu_Premade["ARMORY_URL"].disabled,
    },
    {
		text = LFG_LIST_REPORT_FOR,
		hasArrow = true,
		notCheckable = true,
		menuList = {
			{
				text = LFG_LIST_BAD_PLAYER_NAME,
				notCheckable = true,
				func = function(_, id, memberIdx) C_LFGList.ReportApplicant(id, "badplayername", memberIdx); end,
				arg1 = nil, --Applicant ID goes here
				arg2 = nil, --Applicant Member index goes here
			},
			{
				text = LFG_LIST_BAD_DESCRIPTION,
				notCheckable = true,
				func = function(_, id) C_LFGList.ReportApplicant(id, "lfglistappcomment"); end,
				arg1 = nil, --Applicant ID goes here
			},
		},
	},
	{
		text = IGNORE_PLAYER,
		notCheckable = true,
		func = function(_, name, applicantID) AddIgnore(name); C_LFGList.DeclineApplicant(applicantID); end,
		arg1 = nil, --Player name goes here
		arg2 = nil, --Applicant ID goes here
		disabled = nil, --Disabled if we don't have a name yet
	},
	{
		text = CANCEL,
		notCheckable = true,
	},
};

-- Interface\FrameXML\LFGList.lua line 2763
function LFGListUtil_GetApplicantMemberMenu(applicantID, memberIdx)
    local name, class, localizedClass, level, itemLevel, tank, healer, damage, assignedRole = C_LFGList.GetApplicantMemberInfo(applicantID, memberIdx);
    local id, status, pendingStatus, numMembers, isNew, comment = C_LFGList.GetApplicantInfo(applicantID);
    LFG_LIST_APPLICANT_MEMBER_MENU[1].text = name or " ";
    LFG_LIST_APPLICANT_MEMBER_MENU[2].arg1 = name;
    LFG_LIST_APPLICANT_MEMBER_MENU[2].disabled = not name or (status ~= "applied" and status ~= "invited");
	
	-------------------------------------------------------
	-- add our menu(s)
    LFG_LIST_APPLICANT_MEMBER_MENU[3].arg1 = name;
    LFG_LIST_APPLICANT_MEMBER_MENU[3].disabled = not name or (status ~= "applied" and status ~= "invited");
	-------------------------------------------------------
	
    LFG_LIST_APPLICANT_MEMBER_MENU[4].menuList[1].arg1 = applicantID;
    LFG_LIST_APPLICANT_MEMBER_MENU[4].menuList[1].arg2 = memberIdx;
    LFG_LIST_APPLICANT_MEMBER_MENU[4].menuList[2].arg1 = applicantID;
    LFG_LIST_APPLICANT_MEMBER_MENU[4].menuList[2].disabled = (comment == "");
    LFG_LIST_APPLICANT_MEMBER_MENU[5].arg1 = name;
    LFG_LIST_APPLICANT_MEMBER_MENU[5].arg2 = applicantID;
    LFG_LIST_APPLICANT_MEMBER_MENU[5].disabled = not name;
    return LFG_LIST_APPLICANT_MEMBER_MENU;
end